-- Parsley is Atomicly Delicious.applescript
-- Parsley is Atomicly Delicious

--  Created by James Huston on 2/9/06.
--  Licensed under the GNU GPL: http://www.gnu.org/licenses/gpl.txt.

property processing_files : false -- property to set when processing files
property batch_encode : null -- property to set when batch encoding
property video_kind : "tvshow" -- set a default video kind
property file_list_table : null -- global property for the file list table
property artwork_table : null -- global property for the artwork table
property batch_edit_num : 0 -- global property for total batch editing files
property current_edit_num : 0 -- global property for current file of batch editing
property batch_file_paths : {} -- set an empty list for file paths when batch editing
property movie_file_types : {"m4a", "m4p", "m4b", "m4v", "mp4", "3gp"} -- valid MPEG4 file types
property artwork_file_types : {"jpg", "jpeg", "png"} -- valid artwork file types
property new_file_path : "" -- file path of the new file
property new_file_replaced : "" -- a flag to see if the new file replaces the edited file when renaming
property random_artwork_dir : "/tmp/" -- needed to filter out extracted artwork
property file_has_artwork : false
property open_panel_type : ""
property new_file_ep_num : ""

on clicked theObject
	try
		
		if name of theObject is "but-choose" then -- select MPEG4 files to populate the file list table
			set open_panel_type to "video"
			set allows multiple selection of open panel to true -- allow the user to pick multiple files
			set name of open panel to localized string "OPEN_PANEL" from table "Localized" -- name the open panel form the localizable table
			display open panel for file types movie_file_types attached to window "win-main" -- display the open panel
			
		else if name of theObject is "but-selart" then -- select MPEG4 files to populate the file list table
			set open_panel_type to "image"
			set allows multiple selection of open panel to true -- allow the user to pick multiple files
			set name of open panel to localized string "OPEN_PANEL_IMG" from table "Localized" -- name the open panel form the localizable table
			display open panel for file types artwork_file_types attached to window "win-main" -- display the open panel
			
		else if name of theObject is "but-clear" then -- clear all items in the file list
			delete every data row of file_list_table -- actually delete all the data
			my depopulateTVFields() -- clear TV fields
			my depopulateMovieFields() -- clear movie fields
			my depopulateMusicFields() -- clear music fields
			set enabled of button "but-delete" of window "win-main" to false -- disable the delete button
			delete every data row of artwork_table -- actually delete the data
			try -- clear the disaplyed image. In a try block to suppress errors.
				delete image of image view "iv-artwork" of drawer "draw-artwork" of window "win-main"
			end try
			
		else if name of theObject is "but-clearart" then -- clear all items in the artwork table
			delete every data row of artwork_table -- actually delete the data
			try -- clear the disaplyed image. In a try block to suppress errors.
				delete image of image view "iv-artwork" of drawer "draw-artwork" of window "win-main"
			end try
			
		else if name of theObject is "but-delete" then -- remove the selected file from the file list
			set selected_row to selected row of table view "tv-list" of scroll view "sv-list" of window "win-main"
			tell file_list_table to delete data row selected_row -- delete the selected row
			my changeFileSelection(table view "tv-list" of scroll view "sv-list" of window "win-main") -- update the UI if the selection changes on delete
			
		else if name of theObject is "but-delart" then -- remove the selected artwork
			set selected_row to selected row of table view "tv-artwork" of scroll view "sv-artwork" of drawer "draw-artwork" of window "win-main"
			tell artwork_table to delete data row selected_row -- delete the selected row
			my changeArtworkSelection(table view "tv-artwork" of scroll view "sv-artwork" of drawer "draw-artwork" of window "win-main") -- update the UI if the selection changes on delete
			
		else if name of theObject is "but-process" then -- we're doiong the work now
			set processing_files to true -- set the doing work property to true
			my disableUI() -- disable all the GUI elements
			set contents of progress indicator "pi-long" of window "win-main" to 0 -- set contents of the progress bar to 0
			set visible of progress indicator "pi-long" of window "win-main" to true -- turn on the progress bar
			set contents of text field "tf-progress" of window "win-main" to "Tagging 1 of 1"
			if state of button "but-batch" of window "win-main" is 0 then -- figure out if we're batch editing
				set batch_encode to false -- we aren't
				if (contents of default entry "doDebug" of user defaults as boolean) is true then log "batch: " & batch_encode -- log
				set row_index to selected row of table view "tv-list" of scroll view "sv-list" of window "win-main" -- get the selected file index
				
				if row_index is greater than 0 then -- if we actually selected a row
					set working_file_path to contents of data cell "file_path" of data row row_index of file_list_table -- get the file path of the selected file
					if (contents of default entry "doDebug" of user defaults as boolean) is true then log "file: " & working_file_path -- log
					
					if video_kind = "tvshow" then -- we're tagging a TV Show
						my tagTVShow(working_file_path)
					else if video_kind = "movie" then -- we're tagging a Movie
						my tagMovie(working_file_path)
					else if video_kind = "musicvideo" then -- we're tagging a Music Video
						my tagMusicVideo(working_file_path)
					end if
					
				else -- we haven't selected a row to edit
					my enableUI() -- reenable the UI
					set visible of progress indicator "pi-long" of window "win-main" to false -- turn off the porgress bar
					set processing_files to false -- set working to false
					my showAlert(localized string "CHOOSE_PROMPT" from table "Localized", localized string "CHOOSE_TEXT" from table "Localized", localized string "DEFAULT_BUTTON" from table "Localized", "", "", "win-main", true) -- show alert telling the user to select a file since we are not batch editing
				end if
				
			else if state of button "but-batch" of window "win-main" is 1 then -- we are batch editing
				set batch_encode to true -- so set the property to true
				if (contents of default entry "doDebug" of user defaults as boolean) is true then log "batch: " & batch_encode -- log
				set batch_file_paths to contents of data cell "file_path" of every data row of file_list_table -- build the batch file path list
				set batch_edit_num to count of batch_file_paths -- get the count of total files
				set current_edit_num to 1 -- starting at number 1
				set contents of text field "tf-progress" of window "win-main" to "Tagging 1 of " & batch_edit_num
				
				set working_file_path to item current_edit_num of batch_file_paths -- start work on the first item
				if video_kind = "tvshow" then -- we're tagging a TV Show
					my tagTVShow(working_file_path)
				else if video_kind = "movie" then -- we're tagging a Movie
					my tagMovie(working_file_path)
				else if video_kind = "musicvideo" then -- we're tagging a Music Video
					my tagMusicVideo(working_file_path)
				end if
			end if
		end if
		
	on error the error_message number the error_number -- we had an error
		my cleanTempFiles(false) -- clean up files without deleteing 
		my enableUI() -- reenable the UI
		set visible of progress indicator "pi-long" of window "win-main" to false -- turn off the progress bar
		set processing_files to false -- we aren't doing work
		my showAlert(error_number, error_message, localized string "DEFAULT_ERR_BUT" from table "Localized", "", "", "win-main", true) -- show alert
		
	end try
end clicked

on launched theObject
	set maximum value of progress indicator "pi-long" of window "win-main" to 100 -- set the value of the progress abr
	my cleanTempFiles(true) -- clean up temp files with deleting them
	do shell script "echo ' Progress: >0%--' > /tmp/pad_state.txt;touch /tmp/pad_script.sh" -- set dummy data for temp files
end launched

on will close theObject -- closing the main window
	my cleanTempFiles(true) -- clean up temp files with deletion
	set contents of default entry "WindowBounds" of user defaults to bounds of window "win-main" as list -- save window position
	tell me to quit -- quit when window closes DOES NOT CALL ON WILL QUIT
end will close

on idle theObject
	try
		if processing_files is true then -- check to see if we are working
			if (contents of default entry "doDebug" of user defaults as boolean) is true then do shell script "echo \" Finished writing to temp file.\">>/tmp/pad_state.txt" -- if we are debugging write the complete string
			set is_it_running to do shell script "/usr/bin/grep Finished /tmp/pad_state.txt | /usr/bin/tail -1" -- check to see if AtomicParsley is working
			
			if is_it_running = " Finished writing to temp file." and batch_encode is false then -- we finished all work on a single edit
				set processing_files to false -- no longer doing work
				set contents of progress indicator "pi-long" of window "win-main" to 100 -- set the progress abr to 100
				tell progress indicator "pi-long" of window "win-main" to stop -- stop the progress bar
				my cleanTempFiles(false) -- clean up temp files without deletion
				
				if (contents of default entry "RenameFile" of user defaults as boolean) is true then
					if new_file_replaced is true then -- the renamed file has a non-unique name so we safe delete the exisitng file and rename the new one
						set selected_row to selected row of table view "tv-list" of scroll view "sv-list" of window "win-main"
						set selected_file_path to contents of data cell "file_path" of data row selected_row of file_list_table
						if (contents of default entry "doDebug" of user defaults as boolean) is false then
							tell application "Finder" to delete selected_file_path as POSIX file
							do shell script "mv " & quoted form of new_file_path & " " & quoted form of selected_file_path
						end if
						set new_file_replaced to false
					else if new_file_replaced is false then -- the renamed file has a unique name so we update the UI by removing the selected table row and adding the new one
						set selected_row to selected row of table view "tv-list" of scroll view "sv-list" of window "win-main"
						set selected_file_path to contents of data cell "file_path" of data row selected_row of file_list_table
						tell file_list_table to delete data row selected_row -- delete the selected row
						if (contents of default entry "doDebug" of user defaults as boolean) is false then
							tell application "Finder" to delete selected_file_path as POSIX file
							populateFileListTable(new_file_path as list)
						end if
						set new_file_replaced to false
					end if
				end if
				
				if (contents of default entry "ShowSheet" of user defaults) is 1 then
					my showAlert(localized string "EDIT_DONE_PROMPT" from table "Localized", localized string "EDIT_DONE_TEXT" from table "Localized", localized string "DEFAULT_BUTTON" from table "Localized", "", "", "win-main", true) -- alert that we are finished
				end if
				my enableUI() -- enable the UI
				set visible of progress indicator "pi-long" of window "win-main" to false -- turn off the progress bar
				set contents of text field "tf-progress" of window "win-main" to ""
			else if is_it_running = " Finished writing to temp file." and batch_encode is true then -- finsihed editing file while batch editing
				set is_it_running to "a" -- set a dummy value to running in case we go through the idle cycle again
				my cleanTempFiles(false) -- clean up temp files without deletion
				
				if (contents of default entry "RenameFile" of user defaults as boolean) is true then
					if new_file_replaced is true then -- the renamed file has a non-unique name so we safe delete the exisitng file and rename the new one
						set selected_file_path to contents of data cell "file_path" of data row current_edit_num of file_list_table
						if (contents of default entry "doDebug" of user defaults as boolean) is false then
							tell application "Finder" to delete selected_file_path as POSIX file
							do shell script "mv " & quoted form of new_file_path & " " & quoted form of selected_file_path
						end if
						set new_file_replaced to false
						
					else if new_file_replaced is false then -- the renamed file has a unique name so we update the UI by removing the selected table row and adding the new one
						set selected_file_path to contents of data cell "file_path" of data row current_edit_num of file_list_table
						if (contents of default entry "doDebug" of user defaults as boolean) is false then
							tell file_list_table to delete data row current_edit_num -- delete the selected row
							tell application "Finder" to delete selected_file_path as POSIX file
							populateFileListTable(new_file_path as list)
						end if
						set new_file_replaced to false
					end if
				end if
				
				if current_edit_num  batch_edit_num then -- if the current edit isn't equal to total number of files
					set current_edit_num to current_edit_num + 1 -- increase current by 1
					set contents of progress indicator "pi-long" of window "win-main" to 100 -- set progrss to 100
					set contents of text field "tf-progress" of window "win-main" to "Tagging " & current_edit_num & " of " & batch_edit_num
					set current_file_path to item current_edit_num of batch_file_paths -- get the next file
					if video_kind = "tvshow" then -- we're tagging a TV Show
						my tagTVShow(current_file_path)
					else if video_kind = "movie" then -- we're tagging a Movie
						my tagMovie(current_file_path)
					else if video_kind = "musicvideo" then -- we're tagging a Music Video
						my tagMusicVideo(current_file_path)
					end if
					
				else if current_edit_num = batch_edit_num then -- if the current edit is the same as total number of files
					set processing_files to false -- no more work to do
					set contents of progress indicator "pi-long" of window "win-main" to 100 -- set the progress bar to 100
					tell progress indicator "pi-long" of window "win-main" to stop -- stop the bar
					set contents of text field "tf-progress" of window "win-main" to "" -- set the progress bar to 100
					my cleanTempFiles(false) -- clean up files without deletion
					if (contents of default entry "ShowSheet" of user defaults) is 1 then
						my showAlert(localized string "EDIT_DONE_PROMPT" from table "Localized", localized string "EDIT_DONE_TEXT" from table "Localized", localized string "DEFAULT_BUTTON" from table "Localized", "", "", "win-main", true) -- alert that we are finished
					end if
					my enableUI() -- reenable the UI
					set visible of progress indicator "pi-long" of window "win-main" to false -- turn off bar
				end if
				
			else if is_it_running  " Finished writing to temp file." then -- we aren't quite done with the edit
				set sh_percentage to do shell script "/usr/bin/grep Progress /tmp/pad_state.txt | /usr/bin/strings | /usr/bin/tail -1 | /usr/bin/cut -d \\> -f 2 | /usr/bin/cut -b 1-3" as string -- get the current percentage value 
				if sh_percentage  "" then -- check that we have a file
					if character 2 of sh_percentage is "%" then -- single digit value
						set cur_percentage to character 1 of sh_percentage
						if (contents of default entry "doDebug" of user defaults as boolean) is true then log "cur percentage: " & cur_percentage
					else if character 3 of sh_percentage is "%" then -- double digit value
						set cur_percentage to characters 1 thru 2 of sh_percentage as string
						if (contents of default entry "doDebug" of user defaults as boolean) is true then log "cur percentage: " & cur_percentage
					else -- triple
						set cur_percentage to sh_percentage
						if (contents of default entry "doDebug" of user defaults as boolean) is true then log "cur percentage: " & cur_percentage
					end if
					set cur_percentage to cur_percentage as integer -- set the percentage to a number
					set contents of progress indicator "pi-long" of window "win-main" to cur_percentage -- set the progress abr to current percentage
				end if
				
			end if
		end if
		return 1 -- recheck every second
	on error the error_message number the error_number
		my showAlert(error_number, error_message, localized string "DEFAULT_ERR_BUT" from table "Localized", "", "", "win-main", true) -- we had an error!
		my enableUI()
	end try
end idle

on will quit theObject -- quitting from the menu or command-Q
	try
		my cleanTempFiles(true) -- clean up temp files with deletion
		set contents of default entry "WindowBounds" of user defaults to bounds of window "win-main" as list -- save window position
	end try
end will quit

on choose menu item theObject
	if name of theObject is "mi-prefs" then
		display panel window "win-prefs" attached to window "win-main"
		
	else if (name of current menu item of theObject) is "mi-tvshow" then -- switching to TV Show edit view
		set video_kind to "tvshow" -- set the video kind
		if (contents of default entry "doDebug" of user defaults as boolean) is true then log "video kind: " & video_kind -- log
		tell tab view "tab-type" of window "win-main" -- update the tab view
			set the current tab view item to tab view item "tv-tvshow"
		end tell
		
	else if (name of current menu item of theObject) is "mi-movie" then -- switching to Movie edit view
		set video_kind to "movie" -- set the video kind
		if (contents of default entry "doDebug" of user defaults as boolean) is true then log "video kind: " & video_kind -- log
		tell tab view "tab-type" of window "win-main" -- update the tab view
			set the current tab view item to tab view item "tv-movie"
		end tell
		
	else if (name of current menu item of theObject) is "mi-musicvideo" then -- switching to Music Video edit view
		set video_kind to "musicvideo" -- set the video kind
		if (contents of default entry "doDebug" of user defaults as boolean) is true then log "video kind: " & video_kind -- log
		tell tab view "tab-type" of window "win-main" -- update the tab view
			set the current tab view item to tab view item "tv-musicvideo"
		end tell
	end if
	
end choose menu item

on open theObject -- opening files via the Dock
	try
		set icon_dropped_files to {} -- set up a dummy list
		repeat with i from 1 to count of theObject
			copy POSIX path of item i of theObject to end of icon_dropped_files -- fill the dummy list with the POSIX path of all the items
		end repeat
		my populateFileListTable(icon_dropped_files)
		
	on error the error_message number the error_number -- we errored
		my showAlert(error_number, error_message, localized string "DEFAULT_ERR_BUT" from table "Localized", "", "", "win-main", true)
		my enableUI() -- reenable the UI
	end try
end open

on will finish launching theObject -- populate and read preferences
	make new default entry at end of default entries of user defaults with properties {name:"DeriveEpisodeNum", contents:true}
	make new default entry at end of default entries of user defaults with properties {name:"OverwriteOriginal", contents:false}
	make new default entry at end of default entries of user defaults with properties {name:"doDebug", contents:false}
	make new default entry at end of default entries of user defaults with properties {name:"WindowBounds", contents:""}
	make new default entry at end of default entries of user defaults with properties {name:"ClearFields", contents:false}
	make new default entry at end of default entries of user defaults with properties {name:"DeleteExisting", contents:false}
	make new default entry at end of default entries of user defaults with properties {name:"RenameFile", contents:false}
	make new default entry at end of default entries of user defaults with properties {name:"ShowSheet", contents:true}
	make new default entry at end of default entries of user defaults with properties {name:"RenameFileUseEpNum", contents:false}
	make new default entry at end of default entries of user defaults with properties {name:"SUCheckAtStartup", contents:false}
	set state of button "but-episodenum" of tab view item "tv-tvshow" of tab view "tab-type" of window "win-main" to (contents of default entry "DeriveEpisodeNum" of user defaults)
	set state of button "but-overwrite" of window "win-main" to (contents of default entry "OverwriteOriginal" of user defaults)
	set state of button "but-deldata" of window "win-prefs" to (contents of default entry "DeleteExisting" of user defaults)
	set state of button "but-debug" of window "win-prefs" to (contents of default entry "doDebug" of user defaults)
	set state of button "but-rename" of window "win-prefs" to (contents of default entry "RenameFile" of user defaults)
	set state of button "but-renepnum" of window "win-prefs" to (contents of default entry "RenameFileUseEpNum" of user defaults)
	set state of button "but-sheet" of window "win-prefs" to (contents of default entry "ShowSheet" of user defaults)
	set state of button "but-checkupdates" of window "win-prefs" to (contents of default entry "SUCheckAtStartup" of user defaults)
	if (contents of default entry "RenameFile" of user defaults as boolean) is true then set enabled of button "but-renepnum" of window "win-prefs" to true
	try -- set the window position, in a try block to suppress errors
		set bounds of window "win-main" to (contents of default entry "WindowBounds" of user defaults as list)
	end try
	-- this script makes the atomicparsley binary executable just in case it isn't
	do shell script "chmod +x " & quoted form of (POSIX path of (path to resource "AtomicParsley"))
	-- these three calls work around a bug when AppleScript pulls data from text fields that have not been set
	my depopulateTVFields()
	my depopulateMovieFields()
	my depopulateMusicFields()
	set visible of window "win-main" to true -- open the window
end will finish launching

on awake from nib theObject -- set up tables
	if name of theObject is "tv-list" then
		set file_list_table to data source of table view "tv-list" of scroll view "sv-list" of window "win-main" -- fill the property
		tell file_list_table -- make the table column
			make new data column at the end of the data columns with properties {name:"file_path", sort order:ascending, sort type:alphabetical, sort case sensitivity:case insensitive}
		end tell
		set sorted of file_list_table to true -- sorted
		set sort column of file_list_table to data column "file_path" of file_list_table -- set the sort column
		set data source of theObject to file_list_table
		tell theObject to register drag types {"file names", "string"} -- register drag types
	else if name of theObject is "tv-artwork" then
		set artwork_table to data source of table view "tv-artwork" of scroll view "sv-artwork" of drawer "draw-artwork" of window "win-main" -- fill the proeprty
		tell artwork_table -- make the table column
			make new data column at the end of the data columns with properties {name:"artwork_path", sort order:ascending, sort type:alphabetical, sort case sensitivity:case insensitive}
		end tell
		set data source of theObject to artwork_table
		tell theObject to register drag types {"file names", "string"} -- register drag types
	end if
end awake from nib

on panel ended theObject with result withResult -- the open panel closed
	if open_panel_type is "video" then
		if withResult is 1 then my populateFileListTable(path names of open panel as list) -- populate the file list table
	else if open_panel_type is "image" then
		if withResult is 1 then my populateArtworkTable(path names of open panel as list)
	end if
end panel ended

on drop theObject drag info dragInfo
	if name of theObject is "tv-list" then -- dropping on the file list table
		set file_list to {} -- set up a dummy list
		set temp_list to {} -- set up a dummy list
		set dataTypes to types of pasteboard of dragInfo
		if "file names" is in dataTypes then
			set preferred type of pasteboard of dragInfo to "file names"
			set dropped_files to contents of pasteboard of dragInfo
			my populateFileListTable(dropped_files) -- populate the file list table
		end if
		
	else if name of theObject is "tv-artwork" or name of theObject is "iv-artwork" then -- dropping on the artwork table or image view
		set dataTypes to types of pasteboard of dragInfo
		if "file names" is in dataTypes then
			set preferred type of pasteboard of dragInfo to "file names"
			set current_artwork_list to contents of data cell "artwork_path" of every data row of artwork_table -- pull the data out of the current table
			set dropped_art_files to contents of pasteboard of dragInfo
			copy current_artwork_list to end of dropped_art_files -- copy the current list to then end of the dummy list
			set temp_artwork_list to sortList(dropped_art_files) -- sort the list
			my populateArtworkTable(temp_artwork_list) -- populate the file list table
		end if
	end if
end drop

on selection changed theObject
	try
		if name of theObject is "tv-list" then -- selection in the file list changed
			my changeFileSelection(theObject)
		else if name of theObject is "tv-artwork" then -- selection in the artwork list changed
			my changeArtworkSelection(theObject)
		end if
	on error the error_message number the error_number -- we errored
		my showAlert(error_number, error_message, localized string "DEFAULT_ERR_BUT" from table "Localized", "", "", "win-main", true)
		my enableUI() -- reenable the UI
	end try
end selection changed

on showAlert(dialogText, dialogMessage, defaultButtonTitle, alternateButtonTitle, otherButtonTitle, inWindow, asSheet) -- function to show a diaog with many options
	
	if asSheet is true then
		display alert dialogText as informational message dialogMessage default button defaultButtonTitle alternate button alternateButtonTitle other button otherButtonTitle attached to window inWindow
		
	else if asSheet is false then
		set theReply to display alert dialogText as informational message dialogMessage default button defaultButtonTitle alternate button alternateButtonTitle other button otherButtonTitle
		if (button returned of theReply) is defaultButtonTitle then
			return 1
		else if (button returned of theReply) is alternateButtonTitle then
			return 2
		else if (button returned of theReply) is otherButtonTitle then
			return 3
		end if
	end if
	
end showAlert

on cleanTempFiles(will_be_gone) -- clean up function
	try
		if will_be_gone is false then -- without deletion
			do shell script "echo 'done Progress: >0%---------------------------------------------------------------------------|' > /tmp/pad_state.txt;rm /tmp/pad_script.sh"
		else if will_be_gone is true then -- with deletion
			do shell script "rm /tmp/pad_script.sh;rm /tmp/pad_state.txt"
		end if
	end try
end cleanTempFiles

on tagTVShow(file_to_tag) -- we're tagging a TV Show with this function
	
	if (contents of default entry "doDebug" of user defaults as boolean) is true then log "the file: " & file_to_tag
	set delete_meta_state to contents of default entry "DeleteExisting" of user defaults
	
	set atomic_script to "--stik 'TV Show'" -- set the movie type
	
	if batch_encode is false then -- only set the title when not batch encoding
		set tv_show_title to contents of text field "tf-title" of view of tab view item "tv-tvshow" of tab view "tab-type" of window "win-main"
		if tv_show_title is not "" then
			set atomic_script to atomic_script & " --title " & quoted form of tv_show_title -- set the title
		else if delete_meta_state is 1 and tv_show_title is "" then
			set atomic_script to atomic_script & " --title ''" -- remove the field
		end if
	else if batch_encode is true then -- only set the title when not batch encoding
		set tv_show_title to my trimExtension(file_to_tag)
		set atomic_script to atomic_script & " --title " & quoted form of tv_show_title -- set the title
	end if
	
	set tv_show_name to contents of text field "tf-show" of view of tab view item "tv-tvshow" of tab view "tab-type" of window "win-main"
	if tv_show_name is not "" then
		set atomic_script to atomic_script & " --TVShowName " & quoted form of tv_show_name -- set the show name
	else if delete_meta_state is 1 and tv_show_name is "" then
		set atomic_script to atomic_script & " --TVShowName ''" -- remove the field
	end if
	
	set tv_season_num to contents of text field "tf-season" of view of tab view item "tv-tvshow" of tab view "tab-type" of window "win-main"
	if tv_season_num is not "" then
		set atomic_script to atomic_script & " --TVSeasonNum " & quoted form of tv_season_num -- set the seasun number
	else if delete_meta_state is 1 and tv_season_num is "" then
		set atomic_script to atomic_script & " --TVSeasonNum ''" -- remove the field
	end if
	
	set tv_episode_num to contents of text field "tf-episode" of view of tab view item "tv-tvshow" of tab view "tab-type" of window "win-main"
	if batch_encode is true and tv_episode_num = "" then -- if batch encode is true and no episode number is set
		set tv_episode_num to current_edit_num as integer -- set the episode number to the current number of the file
		set tv_episode_num to padNumber(tv_episode_num as integer)
		set atomic_script to atomic_script & " --TVEpisodeNum " & quoted form of tv_episode_num -- set the episode number
		
	else if batch_encode is true and tv_episode_num  "" then -- if batch encode is true and an episode number is set
		set tv_episode_num to tv_episode_num as integer
		set tv_episode_num to tv_episode_num + (current_edit_num - 1) -- calculate the current episode number
		set tv_episode_num to padNumber(tv_episode_num as integer)
		set atomic_script to atomic_script & " --TVEpisodeNum " & quoted form of tv_episode_num -- set the episode number
		
	else if batch_encode is false then -- if we aren't batch encoding
		if tv_episode_num is not "" then
			set tv_episode_num to padNumber(tv_episode_num as integer)
			set atomic_script to atomic_script & " --TVEpisodeNum " & quoted form of tv_episode_num -- set the episode number
		else if delete_meta_state is 1 and tv_episode_num is "" then
			set atomic_script to atomic_script & " --TVEpisodeNum ''" -- remove the field
		end if
		
	end if
	
	set tv_show_year to contents of text field "tf-year" of view of tab view item "tv-tvshow" of tab view "tab-type" of window "win-main"
	if tv_show_year is not "" then
		set atomic_script to atomic_script & " --year " & quoted form of tv_show_year -- set the year of the episode
	else if delete_meta_state is 1 and tv_show_year is "" then
		set atomic_script to atomic_script & " --year ''" -- remove the field
	end if
	
	set tv_show_genre to contents of text field "tf-genre" of view of tab view item "tv-tvshow" of tab view "tab-type" of window "win-main"
	if tv_show_genre is not "" then
		set atomic_script to atomic_script & " --genre " & quoted form of tv_show_genre -- set the genre
	else if delete_meta_state is 1 and tv_show_genre is "" then
		set atomic_script to atomic_script & " --genre ''" -- remove the field
	end if
	
	set tv_derive_episode_id to state of button "but-episodenum" of view of tab view item "tv-tvshow" of tab view "tab-type" of window "win-main"
	if tv_derive_episode_id is 1 and tv_episode_num  "" and tv_season_num  "" then
		set tv_episode_id to tv_season_num & "x" & tv_episode_num & " " & tv_show_title -- derive the episode #/episode ID
		set atomic_script to atomic_script & " --TVEpisode " & quoted form of tv_episode_id -- apply the episode #
	else if tv_derive_episode_id is 1 and tv_episode_num = "" and tv_episode_num = "" or tv_derive_episode_id is 0 then
		if delete_meta_state is 1 then set atomic_script to atomic_script & " --TVEpisode ''" -- remove the field
	end if
	
	my tagFile(atomic_script, file_to_tag) -- tag the file
	
end tagTVShow

on tagMovie(file_to_tag) -- we're tagging a movie with this function
	
	set atomic_script to "--stik 'Movie'" -- set the movie type
	
	set delete_meta_state to (contents of default entry "DeleteExisting" of user defaults)
	if batch_encode is false then
		set movie_title to contents of text field "tf-movietitle" of view of tab view item "tv-movie" of tab view "tab-type" of window "win-main"
		if movie_title is not "" then
			set atomic_script to atomic_script & " --title " & quoted form of movie_title -- set the title
		else if delete_meta_state is 1 and movie_title is "" then
			set atomic_script to atomic_script & " --title ''" -- remove the field
		end if
	end if
	
	set movie_year to contents of text field "tf-movieyear" of view of tab view item "tv-movie" of tab view "tab-type" of window "win-main"
	if movie_year is not "" then
		set atomic_script to atomic_script & " --year " & quoted form of movie_year -- set the year
	else if delete_meta_state is 1 and movie_year is "" then
		set atomic_script to atomic_script & " --year ''" -- remove the field
	end if
	
	set movie_artist to contents of text field "tf-movieactor" of view of tab view item "tv-movie" of tab view "tab-type" of window "win-main"
	if movie_artist is not "" then
		set atomic_script to atomic_script & " --artist " & quoted form of movie_artist -- set the year
	else if delete_meta_state is 1 and movie_artist is "" then
		set atomic_script to atomic_script & " --artist ''" -- remove the field
	end if
	
	set movie_director to contents of text field "tf-moviedirector" of view of tab view item "tv-movie" of tab view "tab-type" of window "win-main"
	if movie_director is not "" then
		set atomic_script to atomic_script & " --albumArtist " & quoted form of movie_director -- set the year
	else if delete_meta_state is 1 and movie_director is "" then
		set atomic_script to atomic_script & " --albumArtist ''" -- remove the field
	end if
	
	set movie_dvd to contents of text field "tf-moviedvd" of view of tab view item "tv-movie" of tab view "tab-type" of window "win-main"
	if movie_dvd is not "" then
		set atomic_script to atomic_script & " --album " & quoted form of movie_dvd -- set the year
	else if delete_meta_state is 1 and movie_dvd is "" then
		set atomic_script to atomic_script & " --album ''" -- remove the field
	end if
	
	set movie_genre to contents of text field "tf-moviegenre" of view of tab view item "tv-movie" of tab view "tab-type" of window "win-main"
	if movie_genre is not "" then
		set atomic_script to atomic_script & " --genre " & quoted form of movie_genre -- set the year
	else if delete_meta_state is 1 and movie_genre is "" then
		set atomic_script to atomic_script & " --genre ''" -- remove the field
	end if
	
	my tagFile(atomic_script, file_to_tag) -- tag the file
	
end tagMovie

on tagMusicVideo(file_to_tag) -- we're tagging a music video here
	
	set atomic_script to "--stik 'Music Video'"
	set delete_meta_state to (contents of default entry "DeleteExisting" of user defaults)
	
	if batch_encode is false then
		set music_video_title to contents of text field "tf-mvtitle" of view of tab view item "tv-musicvideo" of tab view "tab-type" of window "win-main"
		if music_video_title is not "" then
			set atomic_script to atomic_script & " --title " & quoted form of music_video_title -- set the title
		else if delete_meta_state is 1 and music_video_title is "" then
			set atomic_script to atomic_script & " --title ''" -- remove the field
		end if
	end if
	
	set music_video_artist to contents of text field "tf-mvartist" of view of tab view item "tv-musicvideo" of tab view "tab-type" of window "win-main"
	if music_video_artist is not "" then
		set atomic_script to atomic_script & " --artist " & quoted form of music_video_artist -- set the artist
	else if delete_meta_state is 1 and music_video_artist is "" then
		set atomic_script to atomic_script & " --artist ''" -- remove the field
	end if
	
	set music_video_album to contents of text field "tf-mvalbum" of view of tab view item "tv-musicvideo" of tab view "tab-type" of window "win-main"
	if music_video_album is not "" then
		set atomic_script to atomic_script & " --album " & quoted form of music_video_album -- set the album
	else if delete_meta_state is 1 and music_video_album is "" then
		set atomic_script to atomic_script & " --album ''" -- remove the field
	end if
	
	set music_video_genre to contents of text field "tf-mvgenre" of view of tab view item "tv-musicvideo" of tab view "tab-type" of window "win-main"
	if music_video_genre is not "" then
		set atomic_script to atomic_script & " --genre " & quoted form of music_video_genre -- set the genre
	else if delete_meta_state is 1 and music_video_genre is "" then
		set atomic_script to atomic_script & " --genre ''" -- remove the field
	end if
	
	set music_video_year to contents of text field "tf-mvyear" of view of tab view item "tv-musicvideo" of tab view "tab-type" of window "win-main"
	if music_video_year is not "" then
		set atomic_script to atomic_script & " --year " & quoted form of music_video_year -- set the year
	else if delete_meta_state is 1 and music_video_year is "" then
		set atomic_script to atomic_script & " --year ''" -- remove the field
	end if
	
	my tagFile(atomic_script, file_to_tag) -- tag the file
	
end tagMusicVideo

on tagFile(atomic_script, file_to_tag) -- tag the actual file
	
	set atomicparsley_path to POSIX path of (path to resource "AtomicParsley") -- get the path to the AtomicParsley command
	set delete_meta_state to (contents of default entry "DeleteExisting" of user defaults) -- grab the delete meta preference value
	
	(*********************************************************************
  DESCRIPTION/COMMENT
  **********************************************************************)
	
	if state of button "but-desc" of window "win-main" is 1 then -- we are adding a description/comment
		set MovieComm to contents of text view "tv-tvdesc" of scroll view "sv-tvdesc" of drawer "draw-tvdesc" of window "win-main"
		if MovieComm is not "" then -- it's not empty
			if video_kind is "musicvideo" then -- it's also not a TV Show
				set atomic_script to atomic_script & " --comment " & quoted form of MovieComm -- set the comment
			else if video_kind is "movie" then
				set atomic_script to atomic_script & " --description " & quoted form of MovieComm -- set the episode description
			else if video_kind is "tvshow" then -- it is a TV Show
				set atomic_script to atomic_script & " --description " & quoted form of MovieComm -- set the episode description
			end if
			
		else if delete_meta_state is 1 then -- button clicked but no comment
			if video_kind is "musicvideo" then -- not a TV Show
				set atomic_script to atomic_script & " --comment ''"
			else if video_kind is "movie" then -- is a TV Show
				set atomic_script to atomic_script & " --description ''"
			else if video_kind is "tvshow" then -- is a TV Show
				set atomic_script to atomic_script & " --description ''"
			end if
		end if
		
	else if delete_meta_state is 1 and state of button "but-desc" of window "win-main" is 0 then -- button not clicked at all
		if video_kind is "musicvideo" then -- not a TV Show
			set atomic_script to atomic_script & " --comment ''"
		else if video_kind is "movie" then -- is a TV Show
			set atomic_script to atomic_script & " --description ''"
		else if video_kind is "tvshow" then -- is a TV Show
			set atomic_script to atomic_script & " --description ''"
		end if
	end if
	
	(*********************************************************************
  ARTWORK
  **********************************************************************)
	if state of button "but-artwork" of window "win-main" is 1 then -- check to add artwork
		set artwork_list to {} -- set up an empty list
		set artwork_list to contents of data cell "artwork_path" of every data row of artwork_table
		if (count of artwork_list)  1 then -- if we have artwork loop through it
			repeat with i from 1 to count of artwork_list
				if random_artwork_dir is not in (item i of artwork_list) then
					set atomic_script to atomic_script & " --artwork " & quoted form of (item i of artwork_list) -- set the artwork
				end if
			end repeat
			set artwork_list to {} -- set up an empty list
		else if (count of artwork_list) = 0 then -- if we have artwork loop through it
			set atomic_script to atomic_script & " --artwork REMOVE_ALL " -- remove the field
		end if
	end if
	
	(*********************************************************************
  EXPLICIT
  **********************************************************************)
	if state of button "but-explicit" of window "win-main" is 1 then -- check to flag as explicit
		set atomic_script to atomic_script & " --advisory 'explicit'"
	else if state of button "but-explicit" of window "win-main" is 0 and delete_meta_state is 1 then
		set atomic_script to atomic_script & " --advisory ''"
	end if
	
	(*********************************************************************
  RENAME/OVERWRITE
  **********************************************************************)
	-- renaming is set
	if (contents of default entry "RenameFile" of user defaults as boolean) is true then
		if (contents of default entry "doDebug" of user defaults as boolean) is true then log "Renaming"
		-- grab the file extension
		set file_name_extension to name extension of (info for (file_to_tag as POSIX file))
		
		-- determine which field to grab the title from
		if video_kind is "tvshow" then
			set new_file_name to contents of text field "tf-title" of view of tab view item "tv-tvshow" of tab view "tab-type" of window "win-main"
		else if video_kind is "movie" then
			set new_file_name to contents of text field "tf-movietitle" of view of tab view item "tv-movie" of tab view "tab-type" of window "win-main"
		else if video_kind is "musicvideo" then
			set new_file_name to contents of text field "tf-mvtitle" of view of tab view item "tv-musicvideo" of tab view "tab-type" of window "win-main"
		end if
		
		-- if the title field is blank use the file name minus the extension
		if new_file_name is "" then set new_file_name to my trimExtension(file_to_tag)
		set new_file_name_with_ext to new_file_name & "." & file_name_extension -- set the name with extension
		set file_parent_folder to do shell script "dirname " & quoted form of file_to_tag -- grab the parent folder
		set episode_number to contents of text field "tf-episode" of view of tab view item "tv-tvshow" of tab view "tab-type" of window "win-main"
		
		if (contents of default entry "RenameFileUseEpNum" of user defaults as boolean) is true and video_kind is "tvshow" and episode_number is not "" and batch_encode is false then
			set new_file_ep_num to padNumber(episode_number)
			set new_file_path to file_parent_folder & "/" & new_file_ep_num & " " & sanitizeName(new_file_name_with_ext) -- set the new file path
			
		else if (contents of default entry "RenameFileUseEpNum" of user defaults as boolean) is true and video_kind is "tvshow" and episode_number is "" and batch_encode is true then
			set new_file_ep_num to padNumber(current_edit_num)
			set new_file_path to file_parent_folder & "/" & new_file_ep_num & " " & sanitizeName(new_file_name_with_ext) -- set the new file path
			
		else
			set new_file_path to file_parent_folder & "/" & sanitizeName(new_file_name_with_ext) -- set the new file path
		end if
		
		-- check to see if the file exists
		set file_exists to do shell script "/bin/test -e " & quoted form of new_file_path & " ; echo $?"
		set file_exists to file_exists as integer -- make it a number
		if (contents of default entry "doDebug" of user defaults as boolean) is true then log "Exists: " & file_exists
		
		-- we are replacing files
		if (contents of default entry "OverwriteOriginal" of user defaults as boolean) is true then
			if file_exists is 0 then -- the file exists
				set new_file_path to ""
				set random_seed to random number from 100000 to 999999
				if (contents of default entry "RenameFileUseEpNum" of user defaults as boolean) is true and video_kind is "tvshow" then
					set new_file_name_with_ext to sanitizeName(new_file_ep_num & " " & new_file_name & "-" & random_seed & "." & file_name_extension) -- set the name with extension
				else
					set new_file_name_with_ext to sanitizeName(new_file_name & "-" & random_seed & "." & file_name_extension) -- set the name with extension
				end if
				set new_file_path to file_parent_folder & "/" & new_file_name_with_ext -- set the new file path
				set atomic_script to atomic_script & " --output " & quoted form of new_file_path
				set new_file_replaced to true
			else if file_exists is 1 then -- the file doesn't exists
				set atomic_script to atomic_script & " --output " & quoted form of new_file_path
				set new_file_replaced to false
			end if
			
			-- we aren't replacing files so we seed a random number
		else if (contents of default entry "OverwriteOriginal" of user defaults as boolean) is false then
			if file_exists is 0 then -- the file exists
				set new_file_path to ""
				set random_seed to random number from 100000 to 999999
				if (contents of default entry "RenameFileUseEpNum" of user defaults as boolean) is true and video_kind is "tvshow" then
					set new_file_name_with_ext to sanitizeName(new_file_ep_num & " " & new_file_name & "-" & random_seed & "." & file_name_extension) -- set the name with extension
				else
					set new_file_name_with_ext to sanitizeName(new_file_name & "-" & random_seed & "." & file_name_extension) -- set the name with extension
				end if
				set new_file_path to file_parent_folder & "/" & new_file_name_with_ext -- set the new file path
				set atomic_script to atomic_script & " --output " & quoted form of new_file_path
				set new_file_replaced to false
			else if file_exists is 1 then -- the file doesn't exists
				set atomic_script to atomic_script & " --output " & quoted form of new_file_path
				set new_file_replaced to false
			end if
		end if
		
	else if (contents of default entry "RenameFile" of user defaults as boolean) is false then
		if (contents of default entry "OverwriteOriginal" of user defaults as boolean) is true then
			-- set the overWrite command
			set atomic_script to atomic_script & " --overWrite" as string
			set new_file_replaced to false
		end if
	end if
	
	set full_atomic_script to quoted form of atomicparsley_path & " " & quoted form of file_to_tag & " " & atomic_script & "&>/tmp/pad_state.txt&" -- create the full script command for ease of coding
	
	if (contents of default entry "doDebug" of user defaults as boolean) is true then
		log "script: " & full_atomic_script -- log
	else if (contents of default entry "doDebug" of user defaults as boolean) is false then
		do shell script full_atomic_script
	end if
	
end tagFile

on grabMetaInfo(source_file) -- fill out the fields when selecting a file
	set atomicparsley_path to POSIX path of (path to resource "AtomicParsley") -- get the path to the AtomicParsley command
	set meta_info to do shell script quoted form of atomicparsley_path & " " & quoted form of source_file & " -t" -- build the script
	if meta_info  "" then
		my parseMetaInfo(meta_info, source_file) -- parse the meta info
	end if
end grabMetaInfo

on parseMetaInfo(meta_info, source_file)
	set temp_data_list to {} -- set up a dummy list
	set temp_type_list to {} -- set up a dummy list 
	set atom_data_list to {} -- set up a dummy list
	set temp_list to {} -- set up a dummy list
	set movie_kind to "" -- clear the movie kind
	
	-- Parse the meta list to build it into a nice neat list
	set orig_delims to AppleScript's text item delimiters
	set AppleScript's text item delimiters to return
	set the meta_info_list to every text item of the meta_info
	set AppleScript's text item delimiters to "\""
	repeat with i from 1 to count of meta_info_list
		set current_item to every text item of item i of meta_info_list
		set end of temp_type_list to item 2 of current_item as string -- build the atom type list
	end repeat
	
	set AppleScript's text item delimiters to orig_delims
	set AppleScript's text item delimiters to ": "
	
	repeat with i from 1 to count of meta_info_list
		set current_item to every text item of item i of meta_info_list
		set field_count to count of current_item
		if field_count = 2 then
			set end of temp_data_list to item 2 of current_item as string
		else if field_count > 2 then
			repeat with n from 2 to count of current_item
				set end of temp_list to item n of current_item
			end repeat
			set atom_data_string to temp_list as string
			set end of temp_data_list to atom_data_string as string -- build the atom data list
		end if
	end repeat
	set AppleScript's text item delimiters to orig_delims
	-- End parse the meta list to build it into a nice neat list
	
	
	repeat with i from 1 to count of temp_type_list
		if item i of temp_type_list = "stik" then set movie_kind to item i of temp_data_list -- set the movie kind to figure out what we're setting
		if item i of temp_type_list = "covr" then set file_has_artwork to true
		if (contents of default entry "doDebug" of user defaults as boolean) is true then log "atom type/data: " & item i of temp_type_list & "/" & item i of temp_data_list -- log
		set temp_list to {item i of temp_type_list, item i of temp_data_list}
		set end of atom_data_list to temp_list -- build the nested list
		set temp_list to {}
	end repeat
	
	if movie_kind is "TV Show" then
		my depopulateTVFields() -- clear the fields
		my populateTVFields(atom_data_list) -- populate the fields
	else if movie_kind is "Movie" then
		my depopulateMovieFields() -- clear the fields
		my populateMovieFields(atom_data_list) -- populate the fields
	else if movie_kind is "Music Video" then
		my depopulateMusicFields() -- clear the fields
		my populateMusicFields(atom_data_list) -- populate the fields
	end if
	if file_has_artwork is true then
		my setArtwork(source_file)
	end if
end parseMetaInfo

on setArtwork(source_file)
	set atomicparsley_path to POSIX path of (path to resource "AtomicParsley") -- get the path to the AtomicParsley command
	set base_file_name to name of (info for (source_file as POSIX file))
	set random_artwork_dir to random number from 100000 to 999999
	set temporary_artwork_dir to quoted form of ("/tmp/" & random_artwork_dir & "/")
	do shell script "mkdir -p " & temporary_artwork_dir
	set artwork_script to quoted form of atomicparsley_path & " " & quoted form of source_file & " -e " & temporary_artwork_dir
	set extracted_artwork to do shell script artwork_script
	
	set artwork_files_list to {}
	set orig_delims to AppleScript's text item delimiters
	set AppleScript's text item delimiters to return
	set extracted_artwork_list to every text item of extracted_artwork
	set AppleScript's text item delimiters to ": "
	
	repeat with i from 1 to ((count of extracted_artwork_list) - 1)
		set current_item to every text item of item i of extracted_artwork_list
		set field_count to count of current_item
		if field_count = 2 then
			set end of artwork_files_list to item 2 of current_item as string
		else if field_count > 2 then
			repeat with n from 2 to count of current_item
				set temp_list to {}
				set end of temp_list to item n of current_item
			end repeat
			set artwork_path_string to temp_list as string
			set end of artwork_files_list to artwork_path_string as string -- build the atom data list
		end if
	end repeat
	set AppleScript's text item delimiters to orig_delims
	delete every data row of artwork_table
	my populateArtworkTable(artwork_files_list)
	set state of button "but-artwork" of window "win-main" to 1
	tell drawer "draw-artwork" of window "win-main" to open drawer
end setArtwork

on populateTVFields(meta_info_list) -- populate the TV fields
	set current menu item of popup button "but-videokind" of window "win-main" to menu item "mi-tvshow" of menu of popup button "but-videokind" of window "win-main" -- set the menu to TV Show
	set video_kind to "tvshow"
	repeat with i from 1 to count of meta_info_list -- loop through the meta info list to set the fields
		set cur_meta_info to item i of meta_info_list
		set cur_meta_info_type to item 1 of cur_meta_info
		set cur_meta_info_data to item 2 of cur_meta_info
		if cur_meta_info_type = "tvsh" and state of button "but-batch" of window "win-main" is 0 then
			set contents of text field "tf-show" of tab view item "tv-tvshow" of tab view "tab-type" of window "win-main" to cur_meta_info_data
		else if cur_meta_info_type = "nam" then
			set contents of text field "tf-title" of tab view item "tv-tvshow" of tab view "tab-type" of window "win-main" to cur_meta_info_data
		else if cur_meta_info_type = "tvsn" then
			set contents of text field "tf-season" of tab view item "tv-tvshow" of tab view "tab-type" of window "win-main" to cur_meta_info_data
		else if cur_meta_info_type = "tves" then
			set contents of text field "tf-episode" of tab view item "tv-tvshow" of tab view "tab-type" of window "win-main" to cur_meta_info_data
		else if cur_meta_info_type = "day" then
			set contents of text field "tf-year" of tab view item "tv-tvshow" of tab view "tab-type" of window "win-main" to cur_meta_info_data
		else if cur_meta_info_type = "tven" then
			set contents of default entry "DeriveEpisodeNum" of user defaults to 1
			set state of button "but-episodenum" of tab view item "tv-tvshow" of tab view "tab-type" of window "win-main" to 1
		else if cur_meta_info_type = "gnre" or cur_meta_info_type = "gen" then
			set contents of text field "tf-genre" of tab view item "tv-tvshow" of tab view "tab-type" of window "win-main" to cur_meta_info_data
		else if cur_meta_info_type = "desc" then
			set contents of text view "tv-tvdesc" of scroll view "sv-tvdesc" of drawer "draw-tvdesc" of window "win-main" to cur_meta_info_data
			set state of button "but-desc" of window "win-main" to 1
			tell drawer "draw-tvdesc" of window "win-main" to open drawer
		else if cur_meta_info_type = "rtng" and cur_meta_info_data is equal to "Explicit Lyrics" then
			set state of button "but-explicit" of window "win-main" to 1
		end if
	end repeat
	tell tab view "tab-type" of window "win-main" -- update the tab view
		set the current tab view item to tab view item "tv-tvshow"
	end tell
end populateTVFields

on populateMovieFields(meta_info_list) -- populate the movie fields
	set current menu item of popup button "but-videokind" of window "win-main" to menu item "mi-movie" of menu of popup button "but-videokind" of window "win-main" -- set the menu to TV Show
	set video_kind to "movie"
	repeat with i from 1 to count of meta_info_list -- loop through the meta info list to set the fields
		set cur_meta_info to item i of meta_info_list
		set cur_meta_info_type to item 1 of cur_meta_info
		set cur_meta_info_data to item 2 of cur_meta_info
		if cur_meta_info_type = "nam" and state of button "but-batch" of window "win-main" is 0 then
			set contents of text field "tf-movietitle" of tab view item "tv-movie" of tab view "tab-type" of window "win-main" to cur_meta_info_data
		else if cur_meta_info_type = "day" then
			set contents of text field "tf-movieyear" of tab view item "tv-movie" of tab view "tab-type" of window "win-main" to cur_meta_info_data
		else if cur_meta_info_type = "aART" then
			set contents of text field "tf-moviedirector" of tab view item "tv-movie" of tab view "tab-type" of window "win-main" to cur_meta_info_data
		else if cur_meta_info_type = "ART" then
			set contents of text field "tf-movieactor" of tab view item "tv-movie" of tab view "tab-type" of window "win-main" to cur_meta_info_data
		else if cur_meta_info_type = "alb" then
			set contents of text field "tf-moviedvd" of tab view item "tv-movie" of tab view "tab-type" of window "win-main" to cur_meta_info_data
		else if cur_meta_info_type = "gnre" or cur_meta_info_type = "gen" then
			set contents of text field "tf-moviegenre" of tab view item "tv-movie" of tab view "tab-type" of window "win-main" to cur_meta_info_data
		else if cur_meta_info_type = "com" or cur_meta_info_type = "desc" then
			set contents of text view "tv-tvdesc" of scroll view "sv-tvdesc" of drawer "draw-tvdesc" of window "win-main" to cur_meta_info_data
			set state of button "but-desc" of window "win-main" to 1
			tell drawer "draw-tvdesc" of window "win-main" to open drawer
		else if cur_meta_info_type = "rtng" and cur_meta_info_data is equal to "Explicit Lyrics" then
			set state of button "but-explicit" of window "win-main" to 1
		end if
	end repeat
	tell tab view "tab-type" of window "win-main" -- update the tab view
		set the current tab view item to tab view item "tv-movie"
	end tell
	
end populateMovieFields

on populateMusicFields(meta_info_list) -- populate the music video fields
	set current menu item of popup button "but-videokind" of window "win-main" to menu item "mi-musicvideo" of menu of popup button "but-videokind" of window "win-main" -- set the menu to TV Show
	set video_kind to "musicvideo"
	repeat with i from 1 to count of meta_info_list -- loop through the meta info list to set the fields
		set cur_meta_info to item i of meta_info_list
		set cur_meta_info_type to item 1 of cur_meta_info
		set cur_meta_info_data to item 2 of cur_meta_info
		if cur_meta_info_type = "nam" and state of button "but-batch" of window "win-main" is 0 then
			set contents of text field "tf-mvtitle" of tab view item "tv-musicvideo" of tab view "tab-type" of window "win-main" to cur_meta_info_data
		else if cur_meta_info_type = "ART" then
			set contents of text field "tf-mvartist" of tab view item "tv-musicvideo" of tab view "tab-type" of window "win-main" to cur_meta_info_data
		else if cur_meta_info_type = "alb" then
			set contents of text field "tf-mvalbum" of tab view item "tv-musicvideo" of tab view "tab-type" of window "win-main" to cur_meta_info_data
		else if cur_meta_info_type = "gnre" or cur_meta_info_type = "gen" then
			set contents of text field "tf-mvgenre" of tab view item "tv-musicvideo" of tab view "tab-type" of window "win-main" to cur_meta_info_data
		else if cur_meta_info_type = "day" then
			set contents of text field "tf-mvyear" of tab view item "tv-musicvideo" of tab view "tab-type" of window "win-main" to cur_meta_info_data
		else if cur_meta_info_type = "com" then
			set contents of text view "tv-tvdesc" of scroll view "sv-tvdesc" of drawer "draw-tvdesc" of window "win-main" to cur_meta_info_data
			set state of button "but-desc" of window "win-main" to 1
			tell drawer "draw-tvdesc" of window "win-main" to open drawer
		else if cur_meta_info_type = "rtng" and cur_meta_info_data is equal to "Explicit Lyrics" then
			set state of button "but-explicit" of window "win-main" to 1
		end if
	end repeat
	tell tab view "tab-type" of window "win-main" -- update the tab view
		set the current tab view item to tab view item "tv-musicvideo"
	end tell
end populateMusicFields

on depopulateTVFields() -- clear the TV Show fields
	set contents of every text field of tab view item "tv-tvshow" of tab view "tab-type" of window "win-main" whose editable is true to ""
	set contents of text view "tv-tvdesc" of scroll view "sv-tvdesc" of drawer "draw-tvdesc" of window "win-main" to ""
end depopulateTVFields

on depopulateMovieFields() -- clear the Movie fields
	set contents of every text field of tab view item "tv-movie" of tab view "tab-type" of window "win-main" whose editable is true to ""
	set contents of text view "tv-tvdesc" of scroll view "sv-tvdesc" of drawer "draw-tvdesc" of window "win-main" to ""
end depopulateMovieFields

on depopulateMusicFields() -- clear the Music Video fields
	set contents of every text field of tab view item "tv-musicvideo" of tab view "tab-type" of window "win-main" whose editable is true to ""
	set contents of text view "tv-tvdesc" of scroll view "sv-tvdesc" of drawer "draw-tvdesc" of window "win-main" to ""
end depopulateMusicFields

on changeFileSelection(table_source) -- function to update UI when file list table selection is changed
	set selected_row to selected row of table_source
	if selected_row as integer  0 then -- only update when a new row is selected
		set selected_file_path to contents of data cell "file_path" of data row selected_row of file_list_table
		set baseFileName to trimExtension(selected_file_path)
		set enabled of button "but-delete" of window "win-main" to true
		set contents of text field "tf-title" of view of tab view item "tv-tvshow" of tab view "tab-type" of window "win-main" to baseFileName
		set contents of text field "tf-movietitle" of view of tab view item "tv-movie" of tab view "tab-type" of window "win-main" to baseFileName
		set contents of text field "tf-mvtitle" of view of tab view item "tv-musicvideo" of tab view "tab-type" of window "win-main" to baseFileName
		my grabMetaInfo(selected_file_path) -- grab the meta info of the newly selected file
	else if selected_row as integer = 0 then
		my depopulateTVFields() -- clear TV fields
		my depopulateMovieFields() -- clear movie fields
		my depopulateMusicFields() -- clear music fields
		set enabled of button "but-delete" of window "win-main" to false
		delete every data row of artwork_table -- actually delete the data
		try -- clear the disaplyed image. In a try block to suppress errors.
			delete image of image view "iv-artwork" of drawer "draw-artwork" of window "win-main"
		end try
	end if
end changeFileSelection

on changeArtworkSelection(table_source) -- function to update UI when artwork table selection is changed
	set selected_row to selected row of table_source
	if selected_row as integer  0 then -- only update when a new row is selected
		set selected_file_path to contents of data cell "artwork_path" of data row selected_row of artwork_table
		set enabled of button "but-delart" of drawer "draw-artwork" of window "win-main" to true
		set image of image view "iv-artwork" of drawer "draw-artwork" of window "win-main" to load image selected_file_path -- update the image view with the newly selected artwork
	else if selected_row as integer = 0 then
		try
			delete image of image view "iv-artwork" of drawer "draw-artwork" of window "win-main" -- clear artwork if nothing is selected
			set enabled of button "but-delart" of drawer "draw-artwork" of window "win-main" to false
		end try
	end if
end changeArtworkSelection

on sortList(text_list) -- list sorting function
	set old_delims to AppleScript's text item delimiters
	set AppleScript's text item delimiters to {ASCII character 10} -- always a linefeed
	set list_string to (text_list as string)
	set new_string to do shell script "echo " & quoted form of list_string & " | sort -f"
	set return_list to (paragraphs of new_string)
	set AppleScript's text item delimiters to old_delims
	return return_list
end sortList

on disableUI() -- function to disable UI elements
	set enabled of every button of window "win-main" to false
	set enabled of every button of drawer "draw-artwork" of window "win-main" to false
end disableUI

on enableUI() -- function to enable UI elements
	set selected_row to selected row of table view "tv-list" of scroll view "sv-list" of window "win-main"
	if selected_row is greater than 0 then
		set enabled of every button of window "win-main" to true
	else
		set enabled of every button of window "win-main" whose name is not "but-delete" to true
	end if
	set selected_row to selected row of table view "tv-artwork" of scroll view "sv-artwork" of drawer "draw-artwork" of window "win-main"
	if selected_row is greater than 0 then
		set enabled of every button of drawer "draw-artwork" of window "win-main" to true
	else
		set enabled of every button of drawer "draw-artwork" of window "win-main" whose name is not "but-delart" to true
	end if
	set visible of every progress indicator of window "win-main" to false
end enableUI

on populateArtworkTable(temp_artwork_list) -- function to populate the artwork list table from drag and drop
	set update views of artwork_table to false
	set artwork_list to {} -- set up the joined list
	set temp_list to {}
	
	repeat with i from 1 to count of temp_artwork_list -- loop to filter only valid image types
		set current_file to item i of temp_artwork_list
		if current_file  "" then
			if (name extension of (info for current_file as POSIX file)) is in artwork_file_types then -- if its a valid image type
				set temp_list to {artwork_path:current_file} -- build the array
				copy temp_list to end of artwork_list
				set temp_list to {}
			end if
		end if
	end repeat
	
	delete every data row of artwork_table
	append artwork_table with artwork_list -- populate the table
	set update views of artwork_table to true
	set artwork_list to {}
end populateArtworkTable

on populateFileListTable(file_list) -- function to populate the file list from Open Panel, Drag and Drop on table and Drag and Drop on icon
	set update views of file_list_table to false
	set joined_list to {} -- set up the joined list
	set temp_list to {} -- set up a temp copy list
	set current_list to contents of data cell "file_path" of every data row of file_list_table -- pull the data out of the current table
	copy current_list to end of file_list -- copy the current list to then end of the dummy list
	set joined_file_list to sortList(file_list) -- sort the dummy list
	
	repeat with i from 1 to count of joined_file_list -- loop through to filter only valid movie types
		set current_file to item i of joined_file_list
		if current_file  "" then -- some how we get empty items
			if (name extension of (info for current_file as POSIX file)) is in movie_file_types then -- if the extension is in movie types
				set temp_list to {file_path:current_file} -- build the array
				copy temp_list to end of joined_list -- build the list
				set temp_list to {}
			end if
		end if
	end repeat
	
	delete every data row of file_list_table -- delete the data in the files list
	append file_list_table with joined_list -- populate the file list table
	set file_list to {} -- empty the dummy list
	set joined_list to {} -- empty the joined list
	set enabled of button "but-process" of window "win-main" to true -- enable the process button
	set update views of file_list_table to true
end populateFileListTable

on trimExtension(file_path) -- gets the non-extension name of a file
	set file_path to file_path as string
	if (character 1 of file_path is "/") then
		set alias_file_path to file_path as POSIX file
	else if (character 1 of file_path is not "/") then
		set alias_file_path to file_path as alias
	end if
	set file_info to info for alias_file_path
	set file_name to name of file_info as string
	set name_extension to name extension of file_info
	set extension_count to count of characters of name_extension
	set name_count to count of characters of file_name
	set base_name to characters 1 thru (name_count - (extension_count + 1)) of file_name
	return base_name as string
end trimExtension

on padNumber(the_number) -- pads a number with a leading zero if needed and return it as a string
	set the_number to the_number as integer
	if the_number < 10 then -- if the episode number is less than 10 pad it with a leading 0
		set the_number to the_number as string
		set the_number to "0" & the_number
	else
		set the_number to the_number as string
	end if
	return the_number
end padNumber

on sanitizeName(current_name)
	set search_string_list to {":", "/"}
	set replacement_string to "-"
	repeat with i from 1 to count of search_string_list
		set AppleScript's text item delimiters to the item i of search_string_list
		set the text_item_list to every text item of the current_name
		set AppleScript's text item delimiters to the replacement_string
		set the current_name to the text_item_list as string
	end repeat
	set AppleScript's text item delimiters to ""
	return current_name
end sanitizeName